@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmAuthPeerAuthenticator.cc 11369 2020-12-09 07:25:44Z dwm $
+// @(#) $Id: DwmAuthPeerAuthenticator.cc 11414 2020-12-11 08:36:51Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017, 2020
 //  All rights reserved.
@@ -67,7 +67,7 @@ extern "C" {
 #include "DwmAuthSymCrypto.hh"
 #include "DwmAuthUtils.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.13/src/DwmAuthPeerAuthenticator.cc 11369 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.14/src/DwmAuthPeerAuthenticator.cc 11414 $");
 
 using namespace std;
 using CryptoPP::PSS;
@@ -475,31 +475,10 @@ namespace Dwm {
           string  theirIdCrypted, theirSig;
           if (IDAndSigExchange(fd, peerName, agreement, theirId,
                                theirIdCrypted, theirSig)) {
-            Auth::PublicKeysFile  pubKeysFile;
-            pubKeysFile.Load(_pubKeysPath);
-            Ed25519::PublicKey  theirPubKey;
-            if (pubKeysFile.Find(theirId, theirPubKey)) {
-              string  msgToVerify =
-                to_str(agreement.TheirPublic()) + theirIdCrypted;
-              CryptoPP::ByteQueue  bt;
-              theirPubKey.Save(bt);
-              CryptoPP::ed25519::Verifier  verifier(bt);
-              if (verifier.VerifyMessage((const uint8_t *)msgToVerify.c_str(),
-                                         msgToVerify.size(),
-                                         (const uint8_t *)theirSig.c_str(),
-                                         theirSig.size())) {
-                agreedKey = to_str(agreement.AgreedKey());
-                rc = true;
-              }
-              else {
-                Syslog(LOG_ERR,
-                       "Failed to verify message from %s at %s on fd %d",
-                       theirId.c_str(), peerName.c_str(), fd);
-              }
-            }
-            else {
-              Syslog(LOG_ERR, "Failed to find %s in %s",
-                     theirId.c_str(), _pubKeysPath.c_str());
+            if (Verify(theirId, theirIdCrypted, theirSig,
+                       to_str(agreement.TheirPublic()))) {
+              agreedKey = to_str(agreement.AgreedKey());
+              rc = true;
             }
           }
           else {
@@ -517,6 +496,68 @@ namespace Dwm {
       return rc;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    static bool
+    GetSetSocketNoDelayOption(boost::asio::ip::tcp::socket & sck,
+                              boost::asio::ip::tcp::no_delay & orig,
+                              boost::asio::ip::tcp::no_delay & noDelay)
+    {
+      bool  rc = false;
+      boost::system::error_code  ec;
+      sck.get_option(orig, ec);
+      if (! ec) {
+        sck.set_option(noDelay, ec);
+        if (! ec) {
+          rc = true;
+        }
+        else {
+          Syslog(LOG_ERR, "sck.set_option(no_delay) failed");
+        }
+      }
+      else {
+        Syslog(LOG_ERR, "sck.get_option(no_delay) failed");
+      }
+      return rc;
+    }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool
+    PeerAuthenticator::Verify(const string & theirId,
+                              const string & theirIdCrypted,
+                              const string & theirSig,
+                              const string & theirPublic) const
+    {
+      bool  rc = false;
+      Auth::PublicKeysFile  pubKeysFile;
+      pubKeysFile.Load(_pubKeysPath);
+      Ed25519::PublicKey  theirPubKey;
+      if (pubKeysFile.Find(theirId, theirPubKey)) {
+        string  msgToVerify = theirPublic + theirIdCrypted;
+        CryptoPP::ByteQueue  bt;
+        theirPubKey.Save(bt);
+        CryptoPP::ed25519::Verifier  verifier(bt);
+        if (verifier.VerifyMessage((const uint8_t *)msgToVerify.c_str(),
+                                   msgToVerify.size(),
+                                   (const uint8_t *)theirSig.c_str(),
+                                   theirSig.size())) {
+          rc = true;
+        }
+        else {
+          Syslog(LOG_ERR, "Failed to verify message from %s",
+                 theirId.c_str());
+        }
+      }
+      else {
+        Syslog(LOG_ERR, "Failed to find %s in %s",
+               theirId.c_str(), _pubKeysPath.c_str());
+      }
+      return rc;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -527,39 +568,22 @@ namespace Dwm {
       bool  rc = false;
       theirId.clear();
       agreedKey.clear();
+      
       boost::asio::ip::tcp::no_delay  origNoDelayOption;
-      sck.get_option(origNoDelayOption);
       boost::asio::ip::tcp::no_delay  noDelayOption(true);
-      sck.set_option(noDelayOption);
+      if (! GetSetSocketNoDelayOption(sck, origNoDelayOption, noDelayOption)) {
+        Syslog(LOG_ERR, "Failed to get/set socket no_delay option");
+        return false;
+      }
       Auth::ECDHAgreement  agreement;
       if (ECExchange(sck, agreement)) {
         string  theirIdCrypted, theirSig;
         if (IDAndSigExchange(sck, agreement, theirId,
                              theirIdCrypted, theirSig)) {
-          Auth::PublicKeysFile  pubKeysFile;
-          pubKeysFile.Load(_pubKeysPath);
-          Ed25519::PublicKey  theirPubKey;
-          if (pubKeysFile.Find(theirId, theirPubKey)) {
-            string  msgToVerify =
-              to_str(agreement.TheirPublic()) + theirIdCrypted;
-            CryptoPP::ByteQueue  bt;
-            theirPubKey.Save(bt);
-            CryptoPP::ed25519::Verifier  verifier(bt);
-            if (verifier.VerifyMessage((const uint8_t *)msgToVerify.c_str(),
-                                       msgToVerify.size(),
-                                       (const uint8_t *)theirSig.c_str(),
-                                       theirSig.size())) {
-              agreedKey = to_str(agreement.AgreedKey());
-              rc = true;
-            }
-            else {
-              Syslog(LOG_ERR, "Failed to verify message from %s",
-                     theirId.c_str());
-            }
-          }
-          else {
-            Syslog(LOG_ERR, "Failed to find %s in %s",
-                   theirId.c_str(), _pubKeysPath.c_str());
+          if (Verify(theirId, theirIdCrypted, theirSig,
+                     to_str(agreement.TheirPublic()))) {
+            agreedKey = to_str(agreement.AgreedKey());
+            rc = true;
           }
         }
         else {
@@ -569,7 +593,11 @@ namespace Dwm {
       else {
         Syslog(LOG_ERR, "ECDH exhange failed");
       }
-      sck.set_option(origNoDelayOption);
+
+      if (! GetSetSocketNoDelayOption(sck, noDelayOption, origNoDelayOption)) {
+        Syslog(LOG_ERR, "Failed to get/set socket no_delay option");
+        rc = false;
+      }
       return rc;
     }
 
